require 'fastlane/action'
require 'aliyun/oss'
require_relative '../helper/sxc_aliyun_oss_helper'

module Fastlane
  module Actions
    module SharedValues
      OSS_UPLOAD_DEST_URL = :OSS_UPLOAD_DEST_URL
    end
    class SxcAliyunOssAction < Action
      def self.run(params)
        UI.message("The sxc_aliyun_oss plugin is working!")

        endpoint = params[:endpoint]
        bucket_name = params[:bucket]
        access_key_id = params[:access_key_id]
        access_key_secret = params[:access_key_secret]
        object_key = params[:object_key]

        UI.message "endpoint: #{endpoint}  bucket_name: #{bucket_name}"

        build_file = [
          params[:ipa],
          params[:apk]
        ].detect { |e| !e.to_s.empty? }

        if build_file.nil?
          UI.user_error!("You have to provide a build file")
        end

        UI.message "build_file: #{build_file}"
        # create aliyun oss client
        client = Aliyun::OSS::Client.new(
            endpoint: endpoint,
            access_key_id: access_key_id,
            access_key_secret: access_key_secret
        )

        # get bucket
        bucket = client.get_bucket(bucket_name)
        #
        UI.message "🚀uploading package..."
        bucket.put_object(object_key, :file => build_file)
        Actions.lane_context[SharedValues::OSS_UPLOAD_DEST_URL] = "#{endpoint.sub("://", "://#{bucket_name}.")}/#{object_key}"
        UI.message "✅uploading package success"

      end

      def self.description
        "sxc aliyun oss uploader"
      end

      def self.authors
        ["JimmyDaddy"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        # Optional:
        "upload files to aliyunooss"
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :endpoint,
                                  env_name: "ALIYUN_OSS_ENDPOINT",
                               description: "please provide a endpoint for oss uploader",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :access_key_id,
                                  env_name: "ALIOSS_ACCESS_KEY_ID",
                               description: "access_key_id is required!",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :access_key_secret,
                                  env_name: "ALIOSS_ACCESS_KEY_SECRET",
                               description: "access_key_secret is required!",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :bucket,
                                  env_name: "ALIOSS_BUCKET_NAME",
                               description: "bucket is required!",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :apk,
                                  env_name: "ALIOSS_APK",
                               description: "apk path",
                             default_value: Actions.lane_context[SharedValues::GRADLE_APK_OUTPUT_PATH],
                                  optional: true,
                              verify_block: proc do |value|
                                  UI.user_error!("Couldn't find apk file at path '#{value}'") unless File.exist?(value)
                                end,
                       conflicting_options: [:ipa],
                            conflict_block: proc do |value|
                                    UI.user_error!("You can't use 'apk' and '#{value.key}' options in one run")
                                end),
          FastlaneCore::ConfigItem.new(key: :ipa,
                                  env_name: "ALIOSS_IPA",
                               description: "Path to your IPA file. Optional if you use the _gym_ or _xcodebuild_ action. For Mac zip the .app. For Android provide path to .apk file",
                             default_value: Actions.lane_context[SharedValues::IPA_OUTPUT_PATH],
                                  optional: true,
                              verify_block: proc do |value|
                                  UI.user_error!("Couldn't find ipa file at path '#{value}'") unless File.exist?(value)
                                end,
                       conflicting_options: [:apk],
                            conflict_block: proc do |value|
                                UI.user_error!("You can't use 'ipa' and '#{value.key}' options in one run")
                              end),
          FastlaneCore::ConfigItem.new(key: :object_key,
                                  env_name: "ALIOSS_OBJECT_KEY",
                               description: "object_key is reqiured",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :cname,
                                  env_name: "ALIOSS_CNAME",
                               description: "use cname",
                                  optional: true,
                                      type: Boolean),
        ]
      end

      def self.is_supported?(platform)
        # Adjust this if your plugin only works for a particular platform (iOS vs. Android, for example)
        # See: https://docs.fastlane.tools/advanced/#control-configuration-by-lane-and-by-platform
        #
        # [:ios, :mac, :android].include?(platform)
        true
      end
    end
  end
end
